@@ -13,9 +13,9 @@
 #	* Run the MakeTranslationTable program
 #	  (optional, but highly recommended)
 #         It will create /etc/snare-xlate.conf
-#	* Save this file to /usr/bin/SnareDispatcher.pl
+#	* Save this file to /usr/sbin/SnareDispatcher.pl
 #	* Add the following line to your /etc/auditd.conf file:
-#		dispatcher = /usr/bin/SnareDispatcher.pl
+#		dispatcher = /usr/sbin/SnareDispatcher.pl
 #
 # Notes:
 # Although the items=x field should provide us with a reasonable guestimate
@@ -50,8 +50,7 @@ $DEBUG=0;
 
 ############# User Configurable options.
 
-$XLATE="/usr/share/snarelinux/SnareTranslationTable";
-$WEBSERVER="/usr/share/snarelinux/SnareWebServer.pl";
+$WEBSERVER="/usr/sbin/SnareWebServer.pl";
 $CONFIGFILE="/etc/snare.conf";
 
 # Note: These two values can be configured using the config file.
@@ -64,6 +63,9 @@ $USECRITICALITY=1;
 # Should we manage the audit configuration? Or do we leave it to the native audit subsystem?
 $MANAGEAUDIT=1;
 
+# Should we use regular expressions instead of wildcards?
+$REGEX=0;
+
 # How often should we flush our buffer?
 $FLUSHINTERVAL=10;	# in seconds
 
@@ -72,21 +74,18 @@ $WEBSERVERPORT=6161;
 $ACCESSKEY="";
 $RESTRICT_IP="";
 
+# Where are we sending events?
+$SYSLOGDEST=0;
+
 ############# End user configurable options.
 
-$VERSION="1.0";
+$VERSION="1.1";
 
 $FLUSH=0;
 $FLUSHNOW=-1;
 $CONTINUE=1;
 
 # Open our translation table.
-# This is sorta-optional.
-
-if(! -f "/etc/snare-xlate.conf") {
-	`$XLATE`;
-}
-
 $rc=open(XLATE,"/etc/snare-xlate.conf");
 if($rc) {
 	while($line=<XLATE>) {
@@ -413,7 +412,8 @@ sub SendEvents {
 						$result=0;
 						foreach $alternativecount (keys(%{$Filters{$objectivecount}{$key}{$elementcount}})) {
 							$match=$Filters{$objectivecount}{$key}{$elementcount}{$alternativecount};
-							$element=$elements[$elementcount];
+							#$element=$elements[$elementcount];
+							$element=$elements[0];
 							$negate=$FilterTypes{$objectivecount}{$key};
 							if($DEBUG>2) { LogMsg("Match element $element against $match (negate is $negate)\n"); }
 
@@ -423,7 +423,7 @@ sub SendEvents {
 							} else {
 								$result=Match($element,$match,$negate);
 							}
-							if($result) {
+							if($result && !$negate || !$result && $negate) {
 								# Yay - one of the alternatives matched
 								last;
 							}
@@ -475,21 +475,31 @@ sub SendEvents {
 		# Send this out.
 		if($DEBUG) { LogMsg("OUTPUT: " . $sendstring); }
 
-		SendEvent("$FQDN	LinuxKAudit	$sendstring\n");
+		SendEvent("$FQDN	LinuxKAudit","$sendstring\n");
 	}
 }
 
 sub SendEvent() {
-	my($string)=@_;
+	my($prefix,$string)=@_;
 
 	if(@OUTPUTFILES) {
 		foreach $fp (@OUTPUTFILES) {
-			print $fp $string;
+			print $fp "$prefix\t$string";
 		}
 	}
 	if($OUTPUTNET{"SocketCount"}>0) {
 		for($i=0;$i<$OUTPUTNET{"SocketCount"};$i++) {
-			send($OUTPUTNET{$i}{"Socket"}, $string, 0, $OUTPUTNET{$i}{"PortAddr"});
+			if($OUTPUTNET{$i}{"Port"} == 514) {
+				# Syslog = special
+				# Prepend some syslog gumf. REDRED TODO
+				$sdate=strftime("%b %e %H:%M:%S",localtime);
+				$sprefix="<$SYSLOGDEST> $sdate $FQDN LinuxKAudit";
+				#DMM
+				#$sprefix="<$SYSLOGDEST> LinuxKAudit";
+				send($OUTPUTNET{$i}{"Socket"}, "$sprefix\t$string", 0, $OUTPUTNET{$i}{"PortAddr"});
+			} else {
+				send($OUTPUTNET{$i}{"Socket"}, "$prefix\t$string", 0, $OUTPUTNET{$i}{"PortAddr"});
+			}
 		}
 	}
 
@@ -500,7 +510,7 @@ sub SendEvent() {
 			shift(@EventCache)
 		}
 		if($string ne "") {
-			push(@EventCache,$string);
+			push(@EventCache,"$prefix\t$string");
 		}
 	}
 }
@@ -569,8 +579,18 @@ sub LoadConfig() {
 				$key =~ tr/a-z/A-Z/;
 				if($key eq "USE_CRITICALITY") {
 					$USECRITICALITY=$val;
+				} elsif($key eq "USE_REGEX") {
+					$REGEX=$val;
+				} elsif($key eq "CLIENTNAME") {
+					$FQDN=$val;
 				} elsif($key eq "SET_AUDIT") {
 					$MANAGEAUDIT=$val;
+				} elsif($key eq "SYSLOG_FACILITY") {
+					$SFACILITY=$val;
+					$SFACILITY =~ tr/a-z/A-Z/;
+				} elsif($key eq "SYSLOG_PRIORITY") {
+					$SPRIORITY=$val;
+					$SPRIORITY =~ tr/a-z/A-Z/;
 				}
 			} elsif($section eq "OBJECTIVES") {
 				# Default value.
@@ -604,6 +624,7 @@ sub LoadConfig() {
 					foreach $component (@components) {
 						$AlternativeCount=0;
 						foreach $alternative (split(/,/,$component)) {
+			if($DEBUG) { LogMsg("alternative $key-$ElementCount.$AlternativeCount:$alternative\n"); }
 							$Filters{$ObjectiveCount}{$key}{$ElementCount}{$AlternativeCount}=$alternative;
 							if($check eq "=") {
 								# If the user has specified a non-negate match, and
@@ -620,12 +641,29 @@ sub LoadConfig() {
 						}
 						$ElementCount++;
 					}
-				}			
+				}
+				#@dmtemp = keys %Filters{$ObjectiveCount};
+				#if($DEBUG) { LogMsg("objective: " . @dmtemp . "\n"); }
 
 				$ObjectiveCount++;
 			}
 		}
 	}
+	# Work out syslog facility/priority number:
+	# $SFACILITY $SPRIORITY (all upper)
+	$sfac{"KERNEL"}=0;	$sfac{"USER"}=1;	$sfac{"MAIL"}=2;
+	$sfac{"DAEMON"}=3;	$sfac{"AUTH"}=4;	$sfac{"SYSLOG"}=5;
+	$sfac{"LPR"}=6;		$sfac{"NEWS"}=7;	$sfac{"UUCP"}=8;
+	$sfac{"CRON"}=9;	$sfac{"AUTHPRIV"}=10;	$sfac{"FTP"}=11;
+	$sfac{"LOCAL0"}=16;	$sfac{"LOCAL1"}=17;	$sfac{"LOCAL2"}=18;
+	$sfac{"LOCAL3"}=19;	$sfac{"LOCAL4"}=20;	$sfac{"LOCAL5"}=21;
+	$sfac{"LOCAL6"}=22;
+	$spri{"EMERGENCY"}=0;	$spri{"ALERT"}=1;	$spri{"CRITICAL"}=2;
+	$spri{"ERROR"}=3;	$spri{"WARNING"}=4;	$spri{"NOTICE"}=5;
+	$spri{"INFORMATION"}=6;	$spri{"DEBUG"}=7;
+	
+	$SYSLOGDEST=($sfac{$SFACILITY} << 3) | $spri{$SPRIORITY};
+
 	return(1);
 }
 
@@ -633,6 +671,8 @@ sub SetAudit() {
 	if(%EventsON && $MANAGEAUDIT == 1) {
 		# Clear all audit events
 		`/sbin/auditctl -D`;
+		# Make sure audit is enabled
+		`/sbin/auditctl -e 1`;
 
 		foreach $eventid (keys(%EventsON)) {
 			if($DEBUG) { LogMsg("Turning on event $eventid\n"); }
@@ -670,7 +710,12 @@ sub Match {
 	@matches=split(/,/,$match);
 	$count=0;
 	foreach $tmatch (@matches) {
-		$regex=wildcard($tmatch);
+		if ($REGEX == 1) {
+			$regex=$tmatch;
+			$regex =~ s/[^\\]\//\\\//g;
+		} else {
+			$regex=wildcard($tmatch);
+		}
 		if($DEBUG>1) { LogMsg("Match: $regex against " . $strings[$count]); }
 		if($strings[$count] =~ /$regex/) {
 			$match=!$negate;
@@ -726,6 +771,7 @@ sub OpenOutput() {
 		
 		$OUTPUTNET{$OUTPUTNET{"SocketCount"}}{"Socket"}=$tsocket;
 		$OUTPUTNET{$OUTPUTNET{"SocketCount"}}{"PortAddr"}=$portaddr;
+		$OUTPUTNET{$OUTPUTNET{"SocketCount"}}{"Port"}=$Port;
 		$OUTPUTNET{"SocketCount"}++;
 	}
 	return(0);
@@ -737,8 +783,8 @@ sub OpenOutput() {
 sub wildcard ($) {
 	my $pat = shift or return(".*");
 	my $re = quotemeta $pat;
-	$re =~ s/\\\*/(.+)/g;
-	return($re);
+	$re =~ s/\\\*/(.*)/g;
+	return("^" . $re . "\$");
 }
 
 sub resolve_path {
@@ -853,6 +899,7 @@ sub DumpEvents {
 	}
 }
 
+
 sub LogMsg {
 	my($string)=@_;
 	if(!$string) {
